@@ -3,18 +3,22 @@
 # by Steven Hazel
 
 import urlparse
-import ordereddict
+
+try:
+    from collections import OrderedDict
+except ImportError:
+    from ordereddict import OrderedDict
 
 import tornado.httpclient
 import tornado.httpserver
 
-from monocle import _o, VERSION
-from monocle.deferred import Deferred
+from monocle import _o, Return, VERSION, launch
+from monocle.callback import Callback
 
 
 class HttpException(Exception): pass
 
-HttpHeaders = ordereddict.OrderedDict
+HttpHeaders = OrderedDict
 
 class HttpClient(object):
     def __init__(self):
@@ -27,10 +31,10 @@ class HttpClient(object):
                                              method=method,
                                              headers=headers or {},
                                              body=body)
-        df = Deferred()
-        http_client.fetch(req, df.callback)
-        response = yield df
-        yield response
+        cb = Callback()
+        http_client.fetch(req, cb)
+        response = yield cb
+        yield Return(response)
 
 
 class HttpServer(object):
@@ -39,8 +43,15 @@ class HttpServer(object):
         self.port = port
 
     def _add(self, el):
+        @_o
+        def _handler(request):
+            try:
+                yield launch(self.handler, request)
+            except:
+                yield http_respond(request, 500, {},
+                                   "500 Internal Server Error")
         self._http_server = tornado.httpserver.HTTPServer(
-            self.handler,
+            _handler,
             io_loop=el._tornado_ioloop)
         self._http_server.listen(self.port)
 
@@ -48,9 +59,10 @@ class HttpServer(object):
 @_o
 def http_respond(request, code, headers, content):
     request.write("HTTP/1.1 %s\r\n" % code)
+    headers.setdefault('Server', 'monocle/%s' % VERSION)
+    headers.setdefault('Content-Length', str(len(content)))
     for name, value in headers.iteritems():
         request.write("%s: %s\r\n" % (name, value))
     request.write("\r\n")
     request.write(content)
     request.finish()
-
